// Beipiel MVC, Thomas Zehrer, Oktober/November 2016
//

import java.awt.*;
import java.util.ArrayList;
import javax.swing.*;
import javax.swing.event.*;

// Interface: wird benötigt von TextField, da dieses sich über das ArrayList im Model
// als zu aktualisierendes Steuerelement registrert.
interface Interface 
{
	public void numberValueChanged(Model m, int s);
}

// Das Model. Manipulation der Daten
class Model
{
	private int number;
	private ArrayList<Interface> listeners;		// darin registrieren sich die zu änderenden Objekte
	
	public Model()
	{
		number = 0;
		listeners = new ArrayList<Interface>();
	}
	
	// Getter und Setter 
	public void setNumber(int number)			
	{
		this.number = number;
		fireModelChanged();
	}
	
	public int getNumber()
	{
		return number;
	}
	
	// Damit registrieren sich ausgebende Steuerelemente Views für Änderungen
	public void addSteuerelementeListener(Interface i)
	{
		this.listeners.add(i);
	}
	
	// diese Methode liest die ArrayList aus und aktualisiert über die gemeinsame Methode des 
	// Interfaces die Inhalte der Views
	private void fireModelChanged() 
	{
		for(Interface i : listeners)
		{
			i.numberValueChanged(this, listeners.size());
		}
	}
}

// Textfield, impelementiert Interface um sich über die gemeinesame Methode
// in die ArrayList des Model eintragen zu können.
class ViewJTextField extends JTextField implements Interface
{
	private static final long serialVersionUID = 1L;
	Model m;
	
	public ViewJTextField(Model m)
	{
		this.m = m;
	}

	// Angestossen über fireModelChanged ändert setText AM Steuerelement 
	// letztlich den Inhatl des Textfeldes
	public void numberValueChanged(Model m, int s) 
	{
		setText(Integer.toString(m.getNumber()));
	}
}

// Würde notwendig werden, wenn das Textfeld als Controller den Slider als View controllen soll
//class ViewJSlider extends JSlider implements Interface
//{
//	private static final long serialVersionUID = 1L;
//	Model m;
//	
//	public ViewJSlider(Model m)
//	{
//		this.m = m;
//	}
//
//	public void numberValueChanged(Model m, int s) 
//	{
//		// Noch nicht notwendig, da das Textfeld NICHT den Slider steuert
//	}
//}

// Controller des Sliders. Hier werden Änderungen registriert.
// siehe unten: slider.addChangeListener(sc); bei Changed wird stateChanged(..) aufgerufen.
class SliderController implements ChangeListener
{
	private Model m;

	public SliderController(Model m)
	{
		this.m = m;
	}
	
	public void stateChanged(ChangeEvent arg0) 
	{
		JSlider s = (JSlider)arg0.getSource();
		m.setNumber(s.getValue());
	}
}

// Die Hautpklasse
class MVCSimpleSample extends JFrame
{
	private static final long serialVersionUID = 1L;

	public MVCSimpleSample(String title)
	{
		// Frame vorbereiten
		super(title);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		
		// das Model registrieren
		Model m = new Model();
		
		// Panele vorbereiten
		JPanel p = new JPanel();
		p.setLayout(new GridLayout(0,1));
		
		// View-Textfield erstellen
		ViewJTextField textfield = new ViewJTextField(m);
		textfield.setHorizontalAlignment(JTextField.CENTER);
		textfield.setName("textfield1");
		m.addSteuerelementeListener(textfield);	// ... und im Model registrieren
		
		// Textfield auf die Panele setzen und mit Wert versorgen
		p.add(textfield);
		
		// Control-Slider erstellen
		JSlider slider = new JSlider();		// wäre ViewJSlider wenn das Textfeld den Slider controllen soll.
		slider.setMinimum(1000);
		slider.setMaximum(3000);
		slider.setValue(2000);
		slider.setMajorTickSpacing(500);
		slider.setMinorTickSpacing(100);
		slider.createStandardLabels(2);
		slider.setPaintTicks(true);
		slider.setPaintLabels(true);
		
		// den Conntroller für den Slider erstellen, der Änderungen registriert "stateChanged"
		SliderController sc = new SliderController(m);
		slider.addChangeListener(sc);
				
		// Slider auf die Panele setzen und mit Wert versorgen
		p.add(slider);
		textfield.setText(Integer.toString(slider.getValue()));
				
		// Panele auf Fenster setzen
		this.add(p);
		
		// Frame vollenden
		this.setSize(900,250);
		this.setLocation(200,200);
		this.setVisible(true);	
	}
}

public class SimpleSample 
{
	public static void main(String[] args) 
	{
			// Hier ist der Einstieg, los gehts. Hallo soll der Titel im Window sein.
			new MVCSimpleSample("Hallo");
	}
}
